/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataParsers;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.util.Vector;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

public class DataParserProgressDialog extends JDialog implements Runnable, DataParserListener {

	private JLabel label;
	private int current = 0;
	private int total = 1;
	private ProgressBar progressBar = new ProgressBar();
	private Vector warnings = new Vector();


	public DataParserProgressDialog (JFrame parent) {
		super(parent,"Importing Data...");
		setSize(300,75);
		setLocationRelativeTo(parent);
		
		label = new JLabel("",JLabel.CENTER);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(label,BorderLayout.CENTER);
		getContentPane().add(progressBar,BorderLayout.SOUTH);
		
		Thread t = new Thread(this);
		t.start();
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		setVisible(true);
	}
	
	public void run() {
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {}
	}

	public void dataUpdateProgress(int current, int total, String message) {
		this.current = current;
		this.total = total;
		progressBar.repaint();
		label.setText(message);		
	}

	public void dataExceptionReceived(Exception e) {
		JOptionPane.showMessageDialog(this,e.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
		e.printStackTrace();
		setVisible(false);
		dispose();
	}
	
	public void dataWarningReceived(Exception e) {
		// We just store this warning so we can display all
		// of them at the end
		warnings.add(e);
	}

	public void dataProcessingFinished(boolean b) {
		setVisible(false);

		if (warnings.size() > 0) {
			// We need to display a list of the warnings
			new DataWarningDisplayDialog(this,(Exception [])warnings.toArray(new Exception [0]));
		}
		dispose();
	}
	
	private class ProgressBar extends JPanel {
		
		public void paint (Graphics g) {
			super.paint(g);
			g.setColor(Color.RED);
			g.fillRect(0,0,(int)(getWidth()*((float)current/total)),getHeight());
		}
		
	}
	
}
