package uk.ac.bbsrc.babraham.ChipMonk.DataParsers;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeSet;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;

public class GenericDataParser extends JDialog implements ActionListener, Runnable{
	
	private GenericParser parser;

	private JComboBox delimiters;
	private JComboBox startRow;
	private JComboBox probeCol;
	private JButton importButton;
	private String [] previewData = new String [50];
	private ProbeTableModel model;
	private JTable table;
	private JScrollPane tablePane = null;
	private File probeFile;
	private DataSetsPanel dataSetsPanel;
	private String [] columnList;
	private DataParserProgressDialog progress;
	
	public GenericDataParser (GenericParser parser) {
		super(parser.application(),"Import Data");
		this.parser = parser;
		
		JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getDataLocation());
		chooser.setMultiSelectionEnabled(false);
		chooser.setFileFilter(new FileFilter() {
		
			public String getDescription() {
				return "Text Files";
			}
		
			public boolean accept(File f) {
				if (f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")) {
					return true;
				}
				else {
					return false;
				}
			}
		
		});
		
		int result = chooser.showOpenDialog(this);
		if (result == JFileChooser.CANCEL_OPTION) {
			this.dispose();
			return;
		}

		probeFile = chooser.getSelectedFile();

		// We can now read the first 50 lines from the file
		try {
			BufferedReader br = new BufferedReader(new FileReader(probeFile));
			int x=0;
			String line;
			while (true) {
				line = br.readLine();
				// We could (theoretically) have less than 50 lines of text
				if (line == null) line = "";
				if (x>49) break;
				previewData[x] = new String(line);
				x++;
			}
			br.close();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		

		getContentPane().setLayout(new BorderLayout());
		
		JPanel optionPanel = new JPanel();
		
		optionPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx=0;
		c.gridy=0;
		c.weightx=0.5;
		c.weighty=0.5;
		c.fill = GridBagConstraints.HORIZONTAL;
		
		optionPanel.add(new JLabel("Column Delimiter"),c);
		delimiters = new JComboBox(new String [] {"Tab","Space","Comma"});
		delimiters.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(delimiters,c);
		
		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		optionPanel.add(new JLabel("Start at Row"),c);
		
		String [] rowList = new String[51];
		for (int i=0;i<51;i++) {
			rowList[i] = ""+i;
		}
		
		startRow = new JComboBox(rowList);
		startRow.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(startRow,c);
		
		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		optionPanel.add(new JLabel("Probe ID Col"),c);
		probeCol = new JComboBox();
		probeCol.addActionListener(this);
		c.gridx=1;
		c.weightx=0.1;
		optionPanel.add(probeCol,c);
		

		c.gridx=0;
		c.weightx=0.5;
		c.gridy++;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth=2;
		dataSetsPanel = new DataSetsPanel(this);
		optionPanel.add(dataSetsPanel,c);
		
		c.gridy++;
		importButton = new JButton("Import");
		importButton.setActionCommand("import");
		importButton.setEnabled(false);
		importButton.addActionListener(this);
		optionPanel.add(importButton,c);
		
		getContentPane().add(optionPanel,BorderLayout.EAST);
		model = new ProbeTableModel();
		
		updateTable();

		setSize(800,600);
		setLocationRelativeTo(parser);
		setVisible(true);
	}
	
	private void updateTable () {
		
		// We need to rebuild the list of column numbers.
		columnList = new String [model.getColumnCount()];
		columnList[0] = null;
		for (int i=1;i<model.getColumnCount();i++) {
			columnList[i] = ""+i;
		}
		probeCol.removeAllItems();

		for (int i=0;i<columnList.length;i++) {
			probeCol.addItem(columnList[i]);
		}
		probeCol.validate();
		
		if (tablePane != null) {
			model.fireTableStructureChanged();
			model.fireTableDataChanged();
		}
		else {
			table = new JTable(model);
			MyTableCellRenderer r = new MyTableCellRenderer();
			table.setDefaultRenderer(Object.class, r);
			tablePane = new JScrollPane(table);
			getContentPane().add(tablePane,BorderLayout.CENTER);
			getContentPane().validate();
		}
	}

	private String getDelimiter () {
		if (((String)(delimiters.getSelectedItem())).equals("Tab")) {
			return "\t";
		}
		if (((String)(delimiters.getSelectedItem())).equals("Space")) {
			return " ";
		}
		if (((String)(delimiters.getSelectedItem())).equals("Comma")) {
			return ",";
		}
		return null;
	}
	
	private void checkImportOK () {
		// See if we can enable the import button

		// We need a probe name col to be set
		if (probeCol.getSelectedItem() == null) {
			importButton.setEnabled(false);
			return;
		}

		// We need at least one data definition
		if (! dataSetsPanel.dataSetsDefined()) {
			importButton.setEnabled(false);
			return;
		}
		
		importButton.setEnabled(true);
	}
	
	public void run() {
		// After all the faffing about this is the bit which actually does the import
		// of the new data.
		
		// If we get here then we're doing the actual import
		
		try {
			
			// Get the values for the things we need
			String delimetersValue = getDelimiter();
			int startRowValue = Integer.parseInt((String)startRow.getSelectedItem());
			int probeColValue = Integer.parseInt((String)probeCol.getSelectedItem())-1;
			
			DataSetPair [] dataDefs = dataSetsPanel.getDataSets();
			DataSet [] dataSets = new DataSet[dataDefs.length];
			
//			System.out.println("Dealing with "+dataDefs.length+" data defs");
			
			for (int i=0;i<dataDefs.length;i++) {
				dataSets[i] = new DataSet(dataDefs[i].name);
			}
			
			BufferedReader br  = new BufferedReader(new FileReader(probeFile));
			String line;
			// First skip the header lines
			for (int i=0;i<startRowValue;i++) {
				line = br.readLine();
				if (line == null) {
					throw new Exception ("Ran out of file before skipping all of the header lines");
				}
			}
			
			int maxIndexValue = 0;
			if (probeColValue > maxIndexValue) maxIndexValue = probeColValue;
			for (int i=0;i<dataDefs.length;i++) {
				if (dataDefs[i].controlColumn>maxIndexValue)maxIndexValue=dataDefs[i].controlColumn;
				if (dataDefs[i].experimentColumn>maxIndexValue)maxIndexValue=dataDefs[i].experimentColumn;
			}
			
			ProbeSet p = parser.application().dataCollection().probeSet();
			
			// Now process the rest of the file
			int count = 0;
			int probesAdded = 0;
			int total = parser.application().dataCollection().probeSet().getAllProbes().length;
			while ((line = br.readLine())!= null) {
				if (count%1000 ==0) {
					progress.dataUpdateProgress(count, total, "Processed "+count+" of "+total+" probes");
				}
				++count;
				String [] sections = line.split(delimetersValue);
				
				// Check to see if we've got enough data to work with
				if (maxIndexValue >= sections.length) {
					System.err.println("Needed "+maxIndexValue+" but found only "+sections.length);
					continue; // Skip this line...
				}
				
				
				// Get the probe
				Probe probe = p.getProbeWithName(sections[probeColValue]);

				// Need to check if the probe exists
				if (probe == null) {
					// If it's not there add a warning and move on to the next one.
					progress.dataWarningReceived(new ChipMonkException("Couldn't find a probe called "+sections[probeColValue]));
					continue;
				}
				probesAdded++;
				
//				System.out.println("Found data for probe "+probe.name());
				
				// Add the data for each dataset
				for (int i=0;i<dataDefs.length;i++) {
					double signal = Double.parseDouble(sections[dataDefs[i].experimentColumn]);
					double control = Double.parseDouble(sections[dataDefs[i].controlColumn]);
					dataSets[i].addData(probe, signal, control);
				}
			}
			for (int i=0;i<dataSets.length;i++) {
				parser.application().dataCollection().addDataSet(dataSets[i]);
			}
			
			// We're finished with the file
			br.close();
			
			if (probesAdded != parser.application().dataCollection().probeSet().getAllProbes().length) {
				parser.dataWarningReceived(new ChipMonkException("Only added info for "+probesAdded+" probes when the array has "+parser.application().dataCollection().probeSet().getAllProbes().length));
			}
			
			setVisible(false);
			JOptionPane.showMessageDialog(this,"Imported Data Successfully","Import Succesful",JOptionPane.INFORMATION_MESSAGE);

			dispose();
		}
		catch (Exception e) {
			e.printStackTrace();
			progress.dataExceptionReceived(e);
			JOptionPane.showMessageDialog(this,"Error importing data: "+e.getMessage(),"Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		// We need this line to get the application to update its view.
		setVisible(false);
		parser.application().dataProcessingFinished(true);
		progress.dataProcessingFinished(true);
		dispose();

		
	}
	public void actionPerformed(ActionEvent ae) {
		
		if (ae.getSource() == delimiters) {
			updateTable();
			return;
		}
		
		if (ae.getSource() == startRow) {
			model.fireTableDataChanged();
			return;
		}
		
		if (ae.getSource()!=importButton) {
			checkImportOK();
			return;
		}
		
		// If we get here we need to actually start the data import
		progress = new DataParserProgressDialog(parser.application());
		Thread t = new Thread(this);
		t.start();
		setVisible(false);
		
	}

	private class ProbeTableModel extends AbstractTableModel {

		public int getRowCount() {
			int startRowValue = 1;
			if (startRow.getSelectedItem()!=null) {
				startRowValue = Integer.parseInt((String)startRow.getSelectedItem());
			}
			
			if (startRowValue < 1 || startRowValue > 50) {
				startRowValue = 1;
			}
			
			return 51-startRowValue;
		}
		
		public String getColumnName (int column) {
			if (column==0) {
				return "Row number";
			}
			else {
				return "Col "+column;
			}
		}

		public int getColumnCount() {
			int max = 1;
			for (int i=0;i<previewData.length;i++) {
				String [] sections = previewData[i].split("["+getDelimiter()+"]");
				if (sections.length > max) {
					max = sections.length;
				}
			}
			return max+1;
		}
		


		public Object getValueAt(int r, int c) {
			
			if (c==0) {
				return ""+r;
			}
			
			c-=1;
			
			int startRowValue = 1;
//			if (startRow.getText().length() > 0) {
//				startRowValue = Integer.parseInt(startRow.getText());
//			}
//			
//			if (startRowValue < 1 || startRowValue > 50) {
//				startRowValue = 1;
//			}


			String [] sections = previewData[r+(startRowValue-1)].split("["+getDelimiter()+"]");
			if (sections.length <=c) {
				return null;
			}
			else {
				return sections[c];
			}
		
		}
		
	}
	
	private class DataSetsPanel extends JPanel implements ActionListener {
		
		JList dataSets;
		DefaultListModel model;
		JButton addButton;
		JButton deleteButton;
		GenericDataParser parent;
		
		public DataSetsPanel (GenericDataParser parent) {
			this.parent = parent;
			setLayout(new BorderLayout());
			add(new JLabel("Data Sets",JLabel.CENTER),BorderLayout.NORTH);
			model = new DefaultListModel();
			dataSets = new JList(model);
			add(new JScrollPane(dataSets),BorderLayout.CENTER);
			JPanel buttonPanel = new JPanel();
			addButton = new JButton("Add");
			addButton.addActionListener(this);
			deleteButton = new JButton("Delete");
			deleteButton.addActionListener(this);
			buttonPanel.add(addButton);
			buttonPanel.add(deleteButton);
			add(buttonPanel,BorderLayout.SOUTH);
			
		}
		
		public DataSetPair [] getDataSets () {
			DataSetPair [] d = new DataSetPair [dataSets.getModel().getSize()];
			for (int i=0;i<dataSets.getModel().getSize();i++) {
				d[i] = (DataSetPair)dataSets.getModel().getElementAt(i);
			}
			return d;
		}
		
		public boolean dataSetsDefined () {
			if (model.size()>0) {
				return true;
			}
			return false;
		}

		public void actionPerformed(ActionEvent ae) {

			if (ae.getSource()==deleteButton) {
				int [] o = dataSets.getSelectedIndices();
				for (int i=o.length-1;i>=0;i--) {
					model.removeElementAt(o[i]);
				}
				return;
			}
			if (ae.getSource()==addButton) {
				new AddNewDataPair(parent,model);
				checkImportOK();
			}
			
		}
		
	}
	
	private class AddNewDataPair extends JDialog implements KeyListener,ActionListener {
		DefaultListModel m;
		JTextField name;
		JComboBox exptCol;
		JComboBox controlCol;
		JButton addButton;
		public AddNewDataPair (Component c,DefaultListModel m) {
			super(parser.application());
			setTitle("Add new data pair");
			this.m = m;
			getContentPane().setLayout(new BorderLayout());
			JPanel topPanel = new JPanel();
			topPanel.add(new JLabel("Expt Col"));
			exptCol = new JComboBox(columnList);
			exptCol.addActionListener(this);
			topPanel.add(exptCol);
			controlCol = new JComboBox(columnList);
			controlCol.addActionListener(this);
			topPanel.add(new JLabel("Control Col"));
			topPanel.add(controlCol);
			getContentPane().add(topPanel,BorderLayout.NORTH);
			
			JPanel midPanel = new JPanel();
			midPanel.add(new JLabel("Name"));
			name = new JTextField("",20);
			name.addKeyListener(this);
			midPanel.add(name);
			getContentPane().add(midPanel,BorderLayout.CENTER);
			
			JPanel bottomPanel = new JPanel();
			addButton = new JButton("Add");
			addButton.setEnabled(false);
			addButton.addActionListener(this);
			addButton.setActionCommand("add");
			bottomPanel.add(addButton);
			JButton closeButton = new JButton("Close");
			closeButton.addActionListener(this);
			closeButton.setActionCommand("close");
			bottomPanel.add(closeButton);
			getContentPane().add(bottomPanel,BorderLayout.SOUTH);
			
			setSize(300,140);
			setLocationRelativeTo(c);
			setModal(true);
			setVisible(true);
		}
		public void actionPerformed(ActionEvent ae) {
			if (ae.getSource() instanceof JComboBox) {
				checkOK();
				return;
			}
			if (ae.getActionCommand().equals("close")){
				setVisible(false);
				dispose();
			}
			else if (ae.getActionCommand().equals("add")) {
				m.addElement(new DataSetPair(Integer.parseInt((String)exptCol.getSelectedItem())-1,Integer.parseInt((String)controlCol.getSelectedItem())-1,name.getText()));
				name.setText("");
				controlCol.setSelectedIndex(0);
				exptCol.setSelectedIndex(0);
			}
			else {
				System.err.println("Unknown action "+ae.getActionCommand());
			}
		}
		public void keyPressed(KeyEvent arg0) {
		}
		public void keyReleased(KeyEvent arg0) {
			checkOK();
		}
		public void keyTyped(KeyEvent arg0) {
		}

		public void checkOK () {
			// We need all fields to be filled in
			if (exptCol.getSelectedItem()==null || controlCol.getSelectedItem()==null || name.getText().length()==0) {
				addButton.setEnabled(false);
			}
			else {
				addButton.setEnabled(true);
			}
		}
	}
	
	private class DataSetPair {
		int experimentColumn;
		int controlColumn;
		String name;
		
		public DataSetPair (int experimentColumn, int controlColumn, String name) {
			this.experimentColumn = experimentColumn;
			this.controlColumn = controlColumn;
			this.name = name;
		}
		public String toString () {
			return name;
		}
	}
	
	private class MyTableCellRenderer extends DefaultTableCellRenderer {
		
		// The only point of this class is so we can tell which rows we are ignoring.
		// We do this by selcting the ignored ones so they go grey.
		
		public Component getTableCellRendererComponent (JTable table,Object value,boolean isSelected, boolean hasFocus, int r, int c) {

			if (r<Integer.parseInt((String)startRow.getSelectedItem())) {
				isSelected = true;
			}
			else {
				isSelected = false;
			}
			Component cell = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, r, c);
			
			return cell;
		}
		
		
	}

}
