/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package uk.ac.bbsrc.babraham.ChipMonk.DataParsers;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;

public class GenericParser extends JDialog implements ActionListener, DataParserListener {

	private ChipMonkApplication application;
	private JButton importProbes;
	private JButton importData;
	private Vector warnings = new Vector();
	
	public GenericParser (ChipMonkApplication application) {
		super(application,"Generic Data Import");
		this.application = application;
		
		Container c = getContentPane();
		c.setLayout(new GridBagLayout());
		GridBagConstraints g = new GridBagConstraints();
		g.gridx = 0;
		g.gridy = 0;
		g.weightx = 0.5;
		g.weighty = 0.5;
		g.fill = GridBagConstraints.BOTH;
		
		importProbes = new JButton("Import Probe Data");
		importProbes.setActionCommand("probes");
		importProbes.addActionListener(this);
		c.add(importProbes,g);
		
		g.gridy++;
		importData = new JButton("Import Array Data");
		importData.setActionCommand("data");
		importData.addActionListener(this);
		c.add(importData,g);
		
		
		if (application.dataCollection().probeSet().getAllProbes().length > 0) {
			importProbes.setEnabled(false);
			importProbes.setText("Probe data already imported");
			importData.setEnabled(true);
		}
		else {
			importProbes.setEnabled(true);
			importData.setEnabled(false);
			importData.setText("Probe data must be imported before array data");
		}
		
		setSize(300,150);
		setLocationRelativeTo(application);
		setVisible(true);
		
	}
	
	public ChipMonkApplication application () {
		return application;
	}
	
	public void probesImported () {
		importProbes.setEnabled(false);
		importProbes.setText("Probe data already imported");
		importData.setEnabled(true);			
		importData.setText("Import Array Data");

	}

	public void actionPerformed(ActionEvent ae) {
		
		if (ae.getActionCommand().equals("probes")) {
			GenericProbeParser gpp = new GenericProbeParser(this);
			gpp.addDataParserListener(this);
			gpp.parseProbes();
		}
		else if (ae.getActionCommand().equals("data")) {
			new GenericDataParser(this);
		}
		
	}

	public void dataExceptionReceived(Exception e) {
	}

	public void dataProcessingFinished(boolean requiresNormalisation) {
		if (warnings.size() > 0) {
			new DataWarningDisplayDialog(this,(Exception [])warnings.toArray(new Exception[0]));
			warnings.removeAllElements();
		}
	}

	public void dataUpdateProgress(int current, int total, String message) {
	}

	public void dataWarningReceived(Exception e) {
		warnings.add(e);
	}
	
}
