/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataTypes;

public class DataGroup implements DataStore, Comparable {
	
	private DataSet [] dataSets;
	private String name;
	private DataCollection collection;
	
	public DataGroup (String name, DataSet [] dataSets) {
		this.name = new String(name);
		this.dataSets = dataSets;
	}
	
	public void setCollection (DataCollection collection) {
		this.collection = collection;
	}
	
	public DataSet [] dataSets () {
		return dataSets;
	}
	
	public void setName (String name) {
		this.name = name;
		if (collection != null) {
			collection.dataGroupRenamed(this);
		}
	}
	
	public void setDataSets (DataSet [] sets) {
		dataSets = sets;
		if (collection != null) {
			collection.dataGroupSamplesChanged(this);
		}
	}
	
	public boolean containsDataSet (DataSet s) {
		for (int i=0;i<dataSets.length;i++) {
			if (dataSets[i]==s)
				return true;
		}
		return false;
	}
	
	public void removeDataSet (DataSet s) {
		if (! containsDataSet(s)) return;
		
		DataSet [] newSet = new DataSet[dataSets.length-1];
		int j=0;
		for (int i=0;i<dataSets.length;i++) {
			if (dataSets[i] == s) continue;
			newSet[j] = dataSets[i];
			j++;
		}
		
		dataSets = newSet;
	}

	public double getRawSignalValueForProbe(Probe p) {
		double returnValue = 0;
		for (int i=0;i<dataSets.length;i++) {
			returnValue+=dataSets[i].getRawSignalValueForProbe(p);
		}
		returnValue /= dataSets.length;
		
		return returnValue;
	}

	public double getRawControlValueForProbe(Probe p) {
		double returnValue = 0;
		for (int i=0;i<dataSets.length;i++) {
			returnValue+=dataSets[i].getRawControlValueForProbe(p);
		}
		returnValue /= dataSets.length;
		
		return returnValue;
	}
	
	
	public double getSignalValueForProbe(Probe p) {
		double returnValue = 0;
		for (int i=0;i<dataSets.length;i++) {
			returnValue+=dataSets[i].getSignalValueForProbe(p);
		}
		returnValue /= dataSets.length;
		
		return returnValue;
	}

	public double getControlValueForProbe(Probe p) {
		double returnValue = 0;
		for (int i=0;i<dataSets.length;i++) {
			returnValue+=dataSets[i].getControlValueForProbe(p);
		}
		returnValue /= dataSets.length;
		
		return returnValue;
	}

	public double getRatioForProbe(Probe p) {
		double returnValue = 0;
		for (int i=0;i<dataSets.length;i++) {
			returnValue+=Math.log(dataSets[i].getRatioForProbe(p));
		}
		returnValue /= dataSets.length;
		
		return Math.exp(returnValue);
	}
	
	public String toString () {
		return name();
	}

	public String name() {
		return name;
	}

	public int compareTo(Object o) {
		return name.toLowerCase().compareTo(((DataGroup)o).name.toLowerCase());
	}

}
