/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataTypes;

import java.util.Hashtable;

public class ProbeList implements Comparable {
		
	private Hashtable probeList = new Hashtable();
	private ProbeSet probeSet;
	private String name;
	private String valueName;
	
	public ProbeList (ProbeSet probeSet, String name, String valueName) {
		this.probeSet = probeSet;
		this.name = name;
		this.valueName = valueName;
	}
	
	public void addProbe (Probe p, Double value) {
		if (value==null)
			value = new Double(Double.NaN);
		probeList.put(p,value);
	}
	
	public void setName (String s) {
		this.name = s;
		probeSet.probeListRenamed(this);
	}
	
	public Probe [] getAllProbes () {
		return (Probe [])probeList.keySet().toArray(new Probe [0]);	
	}
		
	public boolean containsProbeWithName (String name) {
		if (probeList.containsKey(probeSet.getProbeWithName(name))) {
			return true;
		}
		return false;
	}
	
	public boolean containsProbe (Probe p) {
		if (probeList.containsKey(p)) {
			return true;
		}
		else {
			return false;
		}
	}
	
	public String getListName () {
		return name;
	}
	
	public String getValueName () {
		if (valueName == null) {
			return "No value";
		}
		return valueName;
	}
	
	public Double getValueForProbe (Probe p) {
		return (Double)probeList.get(p);
	}
	
	public String toString () {
		return name+" ("+probeList.size()+")";
	}

	public int compareTo(Object o) {
		return name.toLowerCase().compareTo(((ProbeList)o).name.toLowerCase());
	}
	
}
