/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.BenjHochFDR;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.OneSampleTTest;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.ProbeTTestValue;
import uk.ac.bbsrc.babraham.ChipMonk.Analysis.Statistics.TwoSampleTTest;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataGroup;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;


public class StatsFilter extends JDialog implements ActionListener, KeyListener, ListSelectionListener, Runnable {

	private ChipMonkApplication application;
	private JList dataList;
	private JTextField cutoff;
	private JCheckBox multiTest;
	private JButton okButton;
	private final double log2 = Math.log(2);
	private FilterProgressDialog progressDialog;
	private String analysisType;

	public StatsFilter (ChipMonkApplication application) {
		super(application,"T-test filter...");
		setSize(500,300);
		setLocationRelativeTo(application);
		
		this.application = application;

		getContentPane().setLayout(new BorderLayout());
		JPanel dataPanel = new JPanel();
		dataPanel.setLayout(new BorderLayout());
		dataPanel.setBorder(BorderFactory.createEmptyBorder(4,4,0,4));

		dataPanel.add(new JLabel("Data Group(s) to Test",JLabel.CENTER),BorderLayout.NORTH);
		
		DefaultListModel dataModel = new DefaultListModel();
		
		DataGroup [] groups = application.dataCollection().getAllDataGroups();
		for (int i=0;i<groups.length;i++) {
			if (groups[i].dataSets().length > 1)
				dataModel.addElement(groups[i]);
		}
		
		// Check to see whether we've got any real data to work with!
		if (dataModel.size() == 0) {
			JOptionPane.showMessageDialog(application, "At least one data group containing >1 data set is required to perform a t-test", "Error", JOptionPane.ERROR_MESSAGE);
			dispose();
			return;
		}
		
		
		dataList = new JList(dataModel);
		dataList.addListSelectionListener(this);
		dataPanel.add(new JScrollPane(dataList),BorderLayout.CENTER);
		
		getContentPane().add(dataPanel,BorderLayout.WEST);
		
		JPanel choicePanel = new JPanel();

		choicePanel.setLayout(new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.weightx=0.2;
		gbc.weighty=0.5;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		
		gbc.gridwidth=2;
		JLabel probeListLabel = new JLabel ("Testing probes in '"+application.dataCollection().probeSet().getActiveProbeListName()+"' ("+application.dataCollection().probeSet().getActiveProbes().length+" probes)",JLabel.CENTER);
		probeListLabel.setFont(new Font("Default",Font.BOLD,12));
		choicePanel.add(probeListLabel,gbc);
		
		gbc.gridwidth=1;
		gbc.gridy++;
		
		choicePanel.add(new JLabel("P-value cutoff",JLabel.RIGHT),gbc);
	
		gbc.gridx++;
		gbc.weightx=0.6;
		
		cutoff = new JTextField("0.01",5);
		cutoff.addKeyListener(this);
		choicePanel.add(cutoff,gbc);
		
		gbc.gridx=0;
		gbc.gridy++;
		gbc.weightx=0.2;
		
		choicePanel.add(new JLabel("Apply multiple testing correction",JLabel.RIGHT),gbc);
		
		gbc.gridx++;
		gbc.weightx=0.6;

		
		multiTest = new JCheckBox();
		multiTest.setSelected(true);
//		multiTest.setEnabled(false);
		choicePanel.add(multiTest,gbc);
		
		getContentPane().add(new JScrollPane(choicePanel),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		okButton = new JButton("Run Filter");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		okButton.setEnabled(false);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void makeSingleProbeList () {
		
		DataGroup g = (DataGroup)dataList.getSelectedValue();
		DataSet [] sets = g.dataSets();
		Probe [] probes = application.dataCollection().probeSet().getActiveProbes();
//		ProbeList newList = new ProbeList(application.dataCollection().probeSet(),"temp","P-value");
		double cutoffValue = Double.parseDouble(cutoff.getText());
		ProbeTTestValue [] newListProbes = new ProbeTTestValue [probes.length];
		
		
		for (int p=0;p<probes.length;p++) {

			if (p % 1000 == 0) {
				progressDialog.updateProgress(p, probes.length);
				try {
					Thread.sleep(10);
				} 
				catch (InterruptedException e1) {

				}
			}

			double [] values = new double [sets.length];
			for (int i=0;i<sets.length;i++) {
				values[i] = Math.log(sets[i].getRatioForProbe(probes[p]))/log2;
			}
			double pValue = 0;
			try {
				pValue = OneSampleTTest.calculatePValue(values,0);
			} catch (ChipMonkException e) {
				e.printStackTrace();
			}
			ProbeTTestValue v = new ProbeTTestValue(probes[p],pValue);
			newListProbes[p] = v;
		}
		
		
		// If multi-testing correction is needed then do it...
		if (multiTest.isSelected()) {
			BenjHochFDR.calculateQValues(newListProbes);
		}
		
		ProbeList newList;
		
		if (multiTest.isSelected()) {
			newList = new ProbeList(application.dataCollection().probeSet(),"","Q-value");
			for (int i=0;i<newListProbes.length;i++) {
				if (newListProbes[i].q <= cutoffValue) {
					newList.addProbe(newListProbes[i].probe,new Double(newListProbes[i].q));
				}
			}
		}
		else {
			newList = new ProbeList(application.dataCollection().probeSet(),"","P-value");
			for (int i=0;i<newListProbes.length;i++) {
				if (newListProbes[i].p <= cutoffValue) {
					newList.addProbe(newListProbes[i].probe,new Double(newListProbes[i].q));
				}
			}
		}

		progressDialog.processingFinished();
		
		// See if any probes actually passed
		if (newList.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes matched the criteria set","Info",JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+newList.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,"Single t-test e<"+cutoffValue);
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}

		newList.setName(groupName);
		
		try {
			application.dataCollection().probeSet().addProbeList(newList);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
		
//		setVisible(false);
//		dispose();
		
	}
	
	private void makeDoubleProbeList () {
		
		Object [] o = dataList.getSelectedValues();
		
		DataGroup [] g = new DataGroup [2];
		g[0] = (DataGroup)o[0];
		g[1] = (DataGroup)o[1];
		
		DataSet [] firstSets = g[0].dataSets();
		DataSet [] secondSets = g[1].dataSets();
		
		Probe [] probes = application.dataCollection().probeSet().getActiveProbes();
		double cutoffValue = Double.parseDouble(cutoff.getText());
		
		ProbeTTestValue [] tTestValues = new ProbeTTestValue[probes.length];
		for (int p=0;p<probes.length;p++) {
			
			if (p % 1000 == 0) {
				progressDialog.updateProgress(p, probes.length);
				try {
					Thread.sleep(10);
				} 
				catch (InterruptedException e1) {

				}
			}
			
			double [] firstValues = new double [firstSets.length];
			for (int i=0;i<firstSets.length;i++) {
				firstValues[i] = Math.log(firstSets[i].getRatioForProbe(probes[p]))/log2;
			}
			
			double [] secondValues = new double [secondSets.length];
			for (int i=0;i<secondSets.length;i++) {
				secondValues[i] = Math.log(secondSets[i].getRatioForProbe(probes[p]))/log2;
			}
						
			double pValue = 0;
			try {
				pValue = TwoSampleTTest.calculatePValue(firstValues,secondValues);
			} catch (ChipMonkException e) {
				e.printStackTrace();
			}
						
			tTestValues[p] = new ProbeTTestValue(probes[p],pValue);
		}

		// Do the multi-testing correction if necessary
		if (multiTest.isSelected()) {
			BenjHochFDR.calculateQValues(tTestValues);
		}
		
		ProbeList newList;
		
		if (multiTest.isSelected()) {
			newList = new ProbeList(application.dataCollection().probeSet(),"","Q-value");
			for (int i=0;i<tTestValues.length;i++) {
				if (tTestValues[i].q <= cutoffValue) {
					newList.addProbe(tTestValues[i].probe,new Double(tTestValues[i].q));
				}
			}
		}
		else {
			newList = new ProbeList(application.dataCollection().probeSet(),"","P-value");
			for (int i=0;i<tTestValues.length;i++) {
				if (tTestValues[i].p <= cutoffValue) {
					newList.addProbe(tTestValues[i].probe,new Double(tTestValues[i].q));
				}
			}
		}

		progressDialog.processingFinished();
		
		// See if any probes actually passed
		if (newList.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes matched the criteria set","Info",JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+newList.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,"Double t-test e<"+cutoffValue);
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}
		newList.setName(groupName);
		
		
		
		try {
			application.dataCollection().probeSet().addProbeList(newList);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
		
//		setVisible(false);
//		dispose();
		
	}
	
	private void checkOK () {
		// Check to see if enough information has been added to allow us to
		// enable the OK button.
		
		// We have to have a dataset selected.
		if (!(dataList.getSelectedIndices().length == 1 || dataList.getSelectedIndices().length == 2)) {
			okButton.setEnabled(false);
			return;
		}
		
		// We need a cutoff
		if (cutoff.getText().length()==0) {
			okButton.setEnabled(false);
			return;
		}
				
		// If we get here then we're good to go
		okButton.setEnabled(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("ok")) {
			if (dataList.getSelectedIndices().length == 1) {
				analysisType = "single";
			}
			else if (dataList.getSelectedIndices().length == 2) {
				analysisType = "double";
			}
			progressDialog = new FilterProgressDialog(this);
			Thread t = new Thread(this);
			t.start();
		}
	}

	public void keyTyped(KeyEvent arg0) {
	}

	public void keyPressed(KeyEvent ke) {

	}

	public void keyReleased(KeyEvent ke) {
		JTextField f = (JTextField)ke.getSource();
		
		char [] c = f.getText().toCharArray();
		
		StringBuffer b = new StringBuffer();
		boolean usedDecimal = false;
		for (int i=0;i<c.length;i++) {
			
			
			if (Character.isDigit(c[i])) {
				b.append(c[i]);
				continue;
			}
			if (c[i] == KeyEvent.VK_PERIOD) {
				if (usedDecimal)
					continue;
				
				b.append(c[i]);
				usedDecimal = true;
				continue;
			}
			f.setText(b.toString());
			break;
		}
		
		
		checkOK();
	}

	public void valueChanged(ListSelectionEvent lse) {
		checkOK();
	}

	public void run() {
		if (analysisType.equals("single"))
			makeSingleProbeList();
		
		else if (analysisType.equals("double"))
			makeDoubleProbeList();		
	}

	
}
