/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Feature;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.FeatureListViewer.FeatureListViewer;


public class FindFeatureDialog extends JDialog implements ActionListener, KeyListener, Runnable{

	private ChipMonkApplication application;
	private JTextField search;
	private JComboBox searchIn;
	private JComboBox featureType;
	private JButton searchButton;
	private FeatureListViewer viewer = null;
	private JScrollPane scroll = null;
	private SearchProgressDialog spd;

	public FindFeatureDialog (ChipMonkApplication application) {
		super(application,"Find Feature...");
		setSize(500,300);
		setLocationRelativeTo(application);
		
		this.application = application;

		getContentPane().setLayout(new BorderLayout());
		
		JPanel choicePanel = new JPanel();
		
		choicePanel.add(new JLabel("Search for "));
		search = new JTextField(15);
		search.addKeyListener(this);
		choicePanel.add(search);
		choicePanel.add(new JLabel(" in "));
		searchIn = new JComboBox(new String [] {"name","all"});
		choicePanel.add(searchIn);
		
		choicePanel.add(new JLabel(" of "));
		
		String [] featureTypes = application.genome().listAvailableFeatureTypes();
		DefaultComboBoxModel model = new DefaultComboBoxModel();
		model.addElement("all");
		for (int i=0;i<featureTypes.length;i++) {
			model.addElement(featureTypes[i]);
		}
		featureType = new JComboBox(model);
		
		choicePanel.add(featureType);
		
		choicePanel.add(new JLabel(" features"));
		
		getContentPane().add(choicePanel,BorderLayout.NORTH);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("close");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		searchButton = new JButton("Search");
		searchButton.setActionCommand("search");
		searchButton.addActionListener(this);
		searchButton.setEnabled(false);
		getRootPane().setDefaultButton(searchButton);
		buttonPanel.add(searchButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	public void run() {
		spd = new SearchProgressDialog(this);
		makeFeatureList();
	}

	private void makeFeatureList () {
		
		if (scroll != null) {
			remove(scroll);
			validate();
		}
		Vector hits = new Vector();
		String query = search.getText().toLowerCase();
		
		boolean searchAll = false;
		if (((String)searchIn.getSelectedItem()).equals("all")) {
			searchAll = true;
		}
		
		
		String [] types;
		
		if (featureType.getSelectedItem().equals("all")) {
			types= application.genome().listAvailableFeatureTypes();
		}
		else {
			types = new String [] {(String)featureType.getSelectedItem()};
		}
		
		Chromosome [] c = application.genome().getAllChromosomes();
		for (int i=0;i<c.length;i++) {
			for (int j=0;j<types.length;j++) {
				spd.updateProgress((i*types.length)+j, c.length*types.length);
				Feature [] f = c[i].getFeaturesForType(types[j]);
				for (int k=0;k<f.length;k++) {
					if (f[k].name().toLowerCase().indexOf(query)>=0) {
						hits.add(f[k]);
						continue;
					}
					if (searchAll) {
						if (f[k].getAllAnnotation().toLowerCase().indexOf(query)>=0) {
							hits.add(f[k]);
						}						
					}
				}
			}
		}
		
		spd.finished();
		
		if (hits.size()>0) {
			viewer = new FeatureListViewer((Feature [])hits.toArray(new Feature[0]),application);			
			scroll = new JScrollPane(viewer);
			add(scroll,BorderLayout.CENTER);
			validate();
		}
		else {
			repaint(); // So we aren't left with a corrupted table showing from a previous search
			JOptionPane.showMessageDialog(this,"No hits found","Search results",JOptionPane.INFORMATION_MESSAGE);
		}
		
	}
	
	private void checkOK () {
		// Check to see if enough information has been added to allow us to
		// enable the OK button.
		
		if (search.getText().length() == 0) {
			searchButton.setEnabled(false);
			return;
		}
		// If we get here then we're good to go
		searchButton.setEnabled(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("close")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("search")) {
			Thread t = new Thread(this);
			t.start();
		}
	}

	public void keyTyped(KeyEvent arg0) {
	}
	public void keyPressed(KeyEvent arg0) {
	}
	public void keyReleased(KeyEvent arg0) {
		checkOK();		
	}	
	
	
	private class SearchProgressDialog extends JDialog implements Runnable {

		private JLabel label;
		private ProgressBar progressBar = new ProgressBar();


		public SearchProgressDialog (JDialog parent) {
			super(parent,"Searching...");
			setSize(300,75);
			setLocationRelativeTo(parent);
			
			label = new JLabel("Searching...",JLabel.CENTER);
			getContentPane().setLayout(new BorderLayout());
			getContentPane().add(label,BorderLayout.CENTER);
			getContentPane().add(progressBar,BorderLayout.SOUTH);
			
			Thread t = new Thread(this);
			t.start();
			setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
			setVisible(true);
		}
		
		public void run() {
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {}
		}
		
		public void finished() {
			setVisible(false);
			dispose();
		}

		public void updateProgress(int current, int total) {
			progressBar.setValues(current, total);
		}
		
	}

	private class ProgressBar extends JPanel {
		
		private int current = 0;
		private int total = 1;
		
		public void setValues (int c, int t) {
			current = c;
			total = t;
			this.repaint();
		}
		public void paint (Graphics g) {
			super.paint(g);
			g.setColor(Color.RED);
			g.fillRect(0,0,(int)(getWidth()*((float)current/total)),getHeight());
		}
		
	}

	
}
