/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Reports;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import com.sun.java.TableSorter;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Feature;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Location;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;
import uk.ac.bbsrc.babraham.ChipMonk.Displays.SmallDoubleCellRenderer;
import uk.ac.bbsrc.babraham.ChipMonk.Preferences.ChipMonkPreferences;


public class AnnotatedListReport extends JDialog implements ActionListener, KeyListener, MouseListener, ItemListener {

	private ChipMonkApplication application;
	private JComboBox annotationPosition;
	private JComboBox annotationType;
	private JComboBox excludes;
	private JComboBox data;
	private JTextField annotationLimit;
	private JButton okButton;
	private JDialog reportDialog;
	private TableModel model;

	public AnnotatedListReport (ChipMonkApplication application) {
		super(application,"Create annotated report...");
		setSize(500,250);
		setLocationRelativeTo(application);
		
		this.application = application;

		getContentPane().setLayout(new BorderLayout());
		
		JPanel choicePanel = new JPanel();
		choicePanel.setLayout(new BoxLayout(choicePanel,BoxLayout.Y_AXIS));
		
		JPanel choicePanel0 = new JPanel();
		JLabel probeListLabel = new JLabel ("Reporting on probes in '"+application.dataCollection().probeSet().getActiveProbeListName()+"' ("+application.dataCollection().probeSet().getActiveProbes().length+" probes)",JLabel.CENTER);
		probeListLabel.setFont(new Font("Default",Font.BOLD,12));
		choicePanel0.add(probeListLabel);
		choicePanel.add(choicePanel0);
		
		JPanel choicePanel1 = new JPanel();
		
		choicePanel1.add(new JLabel("Annotate with "));
		annotationPosition = new JComboBox(new String [] {"closest", "surrounding", "upstream", "downstream", "surrounding or upstream", "surrounding or downstream"});
		annotationPosition.addItemListener(this);
		choicePanel1.add(annotationPosition);
		annotationType = new JComboBox(application.genome().listAvailableFeatureTypes());
		choicePanel1.add(annotationType);
		choicePanel.add(choicePanel1);
		
		
		JPanel choicePanel2 = new JPanel();
		choicePanel2.add(new JLabel("Annotation distance cutoff "));
		annotationLimit = new JTextField("2000",7);
		annotationLimit.addKeyListener(this);
		choicePanel2.add(annotationLimit);
		choicePanel2.add(new JLabel("bp"));
		choicePanel.add(choicePanel2);
		
		JPanel choicePanel3 = new JPanel();
		excludes = new JComboBox(new String [] {"Include","Exclude"});
		choicePanel3.add(excludes);
		choicePanel3.add(new JLabel(" unannotated probes"));
		choicePanel.add(choicePanel3);
		
		getContentPane().add(choicePanel,BorderLayout.CENTER);

		JPanel choicePanel4 = new JPanel();
		data = new JComboBox(new String [] {"Include","Don't include"});
		choicePanel4.add(data);
		choicePanel4.add(new JLabel(" data for currently visible stores"));
		choicePanel.add(choicePanel4);
		
		getContentPane().add(choicePanel,BorderLayout.CENTER);

		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		okButton = new JButton("OK");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void makeReport () {
		
		String annotationTypeValue = (String)annotationType.getSelectedItem();
		int distanceLimit = 0;
		
		// Check what to do with unannotated probes
		boolean includeAll = true;
		if (((String)excludes.getSelectedItem()).equals("Exclude")) {
			includeAll = false;
		}
		
		String annotationPositionValue = (String)annotationPosition.getSelectedItem();
		// We're going to set up a set of booleans which tell us which kinds
		// of relationships we're allowed to look for later.
		boolean surrounding = true;
		boolean upstream = true;
		boolean downstream = true;
		
		if (annotationPositionValue.equals("surrounding")) {
			upstream = false;
			downstream = false;
		}
		else if (annotationPositionValue.equals("surrounding or upstream")) {
			downstream = false;
		}
		else if (annotationPositionValue.equals("surrounding or downstream")) {
			upstream = false;
		}
		else if (annotationPositionValue.equals("upstream")) {
			surrounding = false;
			downstream = false;
		}
		else if (annotationPositionValue.equals("downstream")) {
			surrounding = false;
			upstream = false;
		}
		else if (annotationPositionValue.equals("closest")) {
			// Leave things as they are!
		}
		else {
			System.err.println("Didn't recognise position value '"+annotationPositionValue+"'");
		}
	
		// We only need to worry about a distance cutoff if we're not using
		// surrounding.
		if (!annotationTypeValue.equals("surrounding")) {
			if (annotationLimit.getText().length()> 0) {
				distanceLimit = Integer.parseInt(annotationLimit.getText());
			}
		}
				
		Vector annotations = new Vector();
		
		// Since we're going to be making the annotations on the
		// basis of position we should go through all probes one
		// chromosome at a time.
		
		Chromosome [] chrs = application.genome().getAllChromosomes();
		
		for (int c=0;c<chrs.length;c++) {
			Probe [] probes = application.dataCollection().probeSet().getActiveProbesForChromosome(chrs[c]);
			Feature [] features = chrs[c].getFeaturesForType(annotationTypeValue);
			
			// We can now step through the probes looking for the best feature match
			for (int p=0;p<probes.length;p++) {
				Feature bestFeature = null;
				int closestDistance = 0;
				String relationshipType = "Not found";
				
				for (int f=0;f<features.length;f++) {
					
					if (surrounding) {
						// Quickest check is whether a probe is inside a feature
					
						if (probes[p].position() >= features[f].location().start() && probes[p].position() <= features[f].location().end()) {
							bestFeature = features[f];
							closestDistance = 0;
							relationshipType = "surrounding";
						
							// Once we've found an overlapping feature we quit.
							break;
						}
					}
					
					if (downstream) {
						// Check if the feature is downstream
						// Get the distance to the start
						int d=0;
						if (features[f].location().strand() == Location.FORWARD) {
							d = features[f].location().start() - probes[p].position();
						}
						else {
							d = probes[p].position() - features[f].location().end();
						}
						
						if (d >=0) {

							if (d > distanceLimit || (bestFeature != null && d > closestDistance)) {
								continue;
							}

							// If we make it here this is our best candidate feature so far
							bestFeature = features[f];
							closestDistance = d;
							relationshipType = "downstream";
							continue;
						}
					}

					if (upstream) {
						// Check if the feature is upstream
						// Get the distance to the start
						int d=0;
						if (features[f].location().strand() == Location.FORWARD) {
							d = probes[p].position() - features[f].location().end();
						}
						else {
							d = features[f].location().start() - probes[p].position();
						}
						
						if (d >= 0) {
							// We're the right side of the feature

							if (d > distanceLimit || (bestFeature != null && d > closestDistance)) {
								continue;
							}

							// If we make it here this is our best candidate feature so far
							bestFeature = features[f];
							closestDistance = d;
							relationshipType = "upstream";
							continue;
						}
					}
				
				}
				if (bestFeature == null && (! includeAll)) {
					continue;
				}
				
				annotations.add(new ProbeAnnotation(probes[p],bestFeature,closestDistance,relationshipType));
			}
			
		}
				
		if (includeAll) {
			Probe [] probes = application.dataCollection().probeSet().getActiveProbesForChromosome(null);
			
			for (int p=0;p<probes.length;p++) {
				annotations.add(new ProbeAnnotation(probes[p],null,0,"Not found"));				
			}
			
		}
		
		reportDialog = new JDialog(application,"Report for '"+application.dataCollection().probeSet().getActiveProbeListName()+"'");
		
		DataStore [] stores = new DataStore[0];
		
		if (((String)data.getSelectedItem()).equals("Include")) {
			stores = application.drawnDataStores();
		}
		
		model = new TableSorter(new AnnotationTableModel((ProbeAnnotation [])annotations.toArray(new ProbeAnnotation[0]),application.dataCollection().probeSet().getActiveProbeList(),stores));
		JTable table = new JTable(model);
		table.setDefaultRenderer(Double.class, new SmallDoubleCellRenderer());
		table.addMouseListener(this);
		((TableSorter)model).setTableHeader(table.getTableHeader());

		
		reportDialog.getContentPane().setLayout(new BorderLayout());
		
		reportDialog.getContentPane().add(new JScrollPane(table),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("close");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		JButton saveButton = new JButton("Save");
		saveButton.setActionCommand("save");
		saveButton.addActionListener(this);
		buttonPanel.add(saveButton);
		
		reportDialog.getContentPane().add(buttonPanel,BorderLayout.SOUTH);

		
		reportDialog.setSize(600,400);
		reportDialog.setLocationRelativeTo(application);
		reportDialog.setVisible(true);
		
		setVisible(false);
		dispose();
		
	}
	
	private void checkOK () {
		// Check to see if enough information has been added to allow us to
		// enable the OK button.
		
		// If we get here then we're good to go
		okButton.setEnabled(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("ok")) {
			makeReport();
		}
		// These two actions come from the opened report
		else if (ae.getActionCommand().equals("close")){
			reportDialog.setVisible(false);
			reportDialog.dispose();
		}
		else if (ae.getActionCommand().equals("save")){
			JFileChooser chooser = new JFileChooser(ChipMonkPreferences.getInstance().getSaveLocation());
			chooser.setMultiSelectionEnabled(false);
			chooser.setFileFilter(new FileFilter() {
			
				public String getDescription() {
					return "Text files";
				}
			
				public boolean accept(File f) {
					if (f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")) {
						return true;
					}
					else {
						return false;
					}
				}
			
			});
			
			int result = chooser.showSaveDialog(this);
			if (result == JFileChooser.CANCEL_OPTION) return;

			File file = chooser.getSelectedFile();
			if (! file.getPath().toLowerCase().endsWith(".txt")) {
				file = new File(file.getPath()+".txt");
			}

			// Check if we're stepping on anyone's toes...
			if (file.exists()) {
				int answer = JOptionPane.showOptionDialog(this,file.getName()+" exists.  Do you want to overwrite the existing file?","Overwrite file?",0,JOptionPane.QUESTION_MESSAGE,null,new String [] {"Overwrite and Save","Cancel"},"Overwrite and Save");

				if (answer > 0) {
					return;
				}
			}

			
			try {
				PrintWriter p = new PrintWriter(new FileWriter(file));
				
				int rowCount = model.getRowCount();
				int colCount = model.getColumnCount();
				
				// Do the headers first
				StringBuffer b = new StringBuffer();
				for (int c=0;c<colCount;c++) {
					b.append(model.getColumnName(c));
					if (c+1 != colCount) {
						b.append("\t");
					}
				}
				
				p.println(b);
				
				for (int r=0;r<rowCount;r++) {
					b = new StringBuffer();
					for (int c=0;c<colCount;c++) {
						b.append(model.getValueAt(r,c));
						if (c+1 != colCount) {
							b.append("\t");
						}
					}
					p.println(b);
					
				}
				p.close();
			
			}
				
					
			catch (FileNotFoundException e) {
				JOptionPane.showMessageDialog(application,"Can't save report: e.getMessage()","Error",JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
			catch (IOException e) {
				JOptionPane.showMessageDialog(application,"Can't save report: e.getMessage()","Error",JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
			
		}
		
	}

	public void keyTyped(KeyEvent arg0) {
	}

	public void keyPressed(KeyEvent ke) {

	}

	public void keyReleased(KeyEvent ke) {
		JTextField f = (JTextField)ke.getSource();
		
		char [] c = f.getText().toCharArray();
		
		StringBuffer b = new StringBuffer();
		for (int i=0;i<c.length;i++) {
			
			if (Character.isDigit(c[i])) {
				b.append(c[i]);
				continue;
			}

			f.setText(b.toString());
			break;
		}
		
		
		checkOK();
	}
	
	private class ProbeAnnotation {
		
		public Probe probe;
		public Feature feature;
		public int distance;
		public String orientation;
		
		public ProbeAnnotation (Probe p, Feature f, int d, String o) {
			probe = p;
			feature = f;
			distance = d;
			orientation = o;
		}
	}
	
	private class AnnotationTableModel extends AbstractTableModel  {
		
		private ProbeAnnotation [] data;
		private ProbeList list;
		private DataStore [] stores;
		
		public AnnotationTableModel (ProbeAnnotation [] data, ProbeList list, DataStore [] stores) {
			this.data = data;
			this.list = list;
			this.stores = stores;
		}

		public int getRowCount() {
			return data.length;
		}

		public int getColumnCount() {
			return 9+stores.length;
		}

		public String getColumnName (int c) {
			switch (c) {
			case 0: return "Probe";
			case 1: return "Chromosome";
			case 2: return "Position";
			case 3: if (list != null)return list.getValueName(); else return "No value";
			case 4: return "Feature";
			case 5: return "Description";
			case 6: return "Type";
			case 7: return "Orientation";
			case 8: return "Distance";
			default: return stores[c-9].name();
			}
		}

		public Class getColumnClass (int c) {
			switch (c) {
			case 0: return Object.class;
			case 1: return String.class;
			case 2: return Integer.class;
			case 3: return Double.class;
			case 4: return Object.class;
			case 5: return String.class;
			case 6: return String.class;
			case 7: return String.class;
			case 8: return Integer.class;
			default: return Double.class;
			}
		}

		
		public Object getValueAt(int r, int c) {
			switch (c) {
			case 0 :
				return data[r].probe;
			
			case 1:
				return data[r].probe.chromosome();
				
			case 2:
				return new Integer(data[r].probe.position());
				
			case 3:
				if (list != null) return list.getValueForProbe(data[r].probe);
				else return new Double(Double.NaN);
				
			case 4:
				return data[r].feature;

			case 5:
				if (data[r].feature != null)
					return data[r].feature.description();
				else 
					return "";

			case 6:
				if (data[r].feature != null) 
					return data[r].feature.type();
				else 
					return "";
				
			case 7:
				return data[r].orientation;
			
			case 8:
				return new Integer(data[r].distance);

			default:
				return new Double(stores[c-9].getRatioForProbe(data[r].probe));
			}
		}
		
	}

	public void mouseClicked(MouseEvent me) {
		//We're only interested in double clicks
		if (me.getClickCount() !=2 ) return;
		// This is only linked from the report JTable
		JTable t = (JTable)me.getSource();
		int r = t.getSelectedRow();
		Probe p = (Probe)t.getValueAt(r,0);
		application.displayChromosome(p.chromosome(),p.position());
		application.chromosomeViewer().setView(p.position()-100,p.position()+100);
	}

	public void mousePressed(MouseEvent arg0) {
	}
	public void mouseReleased(MouseEvent arg0) {
	}
	public void mouseEntered(MouseEvent arg0) {
	}
	public void mouseExited(MouseEvent arg0) {
	}
	public void itemStateChanged(ItemEvent arg0) {
		String a = (String)annotationPosition.getSelectedItem();
		
		if (a.equals("surrounding")) {
			annotationLimit.setEnabled(false);
		}
		else {
			annotationLimit.setEnabled(true);
		}
	}
}
