/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Displays.FeatureListViewer;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import com.sun.java.TableSorter;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Feature;

public class FeatureListViewer extends JTable implements MouseListener {
		
	private ChipMonkApplication application;
	
	public FeatureListViewer (Feature [] features, ChipMonkApplication application) {
		super();

		this.application = application;


		String [] headers = new String [] {"Feature","Type","Chr","Start","End"};
		Class [] classes = new Class [] {String.class,String.class,String.class,Integer.class,Integer.class};
		
		Object [][] rowData = new Object [features.length][headers.length];
		
		for (int i=0;i<features.length;i++) {
			rowData[i][0] = features[i];
			rowData[i][1] = features[i].type();
			if (features[i].chromosome() != null) {
				rowData[i][2] = features[i].chromosome().name();
			}
			else {
				rowData[i][1] = "No chr";				
			}
			rowData[i][3] = new Integer(features[i].location().start());
			rowData[i][4] = new Integer(features[i].location().end());
		}

		TableSorter sorter = new TableSorter(new FeatureTableModel(rowData,headers,classes));
		setModel(sorter);
		addMouseListener(this);
		sorter.setTableHeader(getTableHeader());
	}

	private class FeatureTableModel extends AbstractTableModel {

		private Object [][] data;
		private String [] headers;
		private Class [] classes;
		
		public FeatureTableModel (Object [][] data, String [] headers, Class [] classes) {
			super();
			this.data = data;
			this.headers = headers;
			this.classes = classes;
		}
		
		public int getRowCount() {
			return data.length;
		}

		public int getColumnCount() {
			return data[0].length;
		}

		public Object getValueAt(int r, int c) {
			return data[r][c];
		}
		
		public String getColumnName (int c) {
			return headers[c];
		}
		
		public Class getColumnClass (int c) {
			return classes[c];
		}
		
		public boolean isCellEditable (int r, int c) {
			return false;
		}
	}

	public void mouseClicked(MouseEvent me) {
		//We're only interested in double clicks
		if (me.getClickCount() !=2 ) return;
		// This is only linked from the report JTable
		JTable t = (JTable)me.getSource();
		int r = t.getSelectedRow();
		Feature f = (Feature)t.getValueAt(r,0);
		application.displayChromosome(f.chromosome(),f.location().start());
		application.chromosomeViewer().setView(f.location().start(),f.location().end());
	}

	public void mousePressed(MouseEvent arg0) {
	}
	public void mouseReleased(MouseEvent arg0) {
	}
	public void mouseEntered(MouseEvent arg0) {
	}
	public void mouseExited(MouseEvent arg0) {
	}
}
