/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Displays.GenomeViewer;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Genome;

public class ChromosomeDisplay extends JPanel {

	private int maxLen;
	private Chromosome chromosome;
	private GenomeViewer viewer;
	private boolean showView = false;
	private int viewStart = 0;
	private int viewEnd = 0;
	
	public ChromosomeDisplay (Genome genome, Chromosome chromosome, GenomeViewer viewer) {
		maxLen = genome.getLongestChromosomeLength();
		this.chromosome = chromosome;
		this.viewer = viewer;
		addMouseListener(new PanelListener());
	}
	
	public void paintComponent (Graphics g) {
		super.paintComponent(g);
		int xOffset = getWidth()/80;
		if (xOffset > 10) xOffset=10;
		if (xOffset < 1) xOffset = 1;
		
		int yOffset = getHeight()/10;
		if (yOffset > 10) yOffset = 10;
		if (yOffset < 2) yOffset = 2;
		
		int width = getWidth() - (2*xOffset);
		int height = getHeight() - (2*yOffset);
		
		g.setColor(Color.WHITE);
		g.fillRect(0,0,getWidth(),getHeight());
		
		g.setColor(Color.BLUE);
		g.fillRoundRect(xOffset,yOffset,scaleX(width,chromosome.length(),maxLen),height,2,2);
//		g.setColor(Color.BLACK);
//		g.drawRoundRect(xOffset,yOffset,scaleX(width,chromosome.length(),maxLen),height,2,2);
		
		// Draw a box around the selected region if there is one
		if (showView) {
			g.setColor(Color.RED);
			g.drawRect(xOffset+scaleX(width,viewStart,maxLen),1,scaleX(width,viewEnd-viewStart,maxLen),getHeight()-2);
			g.drawRect(xOffset+1+scaleX(width,viewStart,maxLen),2,scaleX(width,viewEnd-viewStart,maxLen)-2,getHeight()-4);
		}
	}

	private int scaleX (int width, double measure, double max) {
		return (int)(width*(measure/max));
	}
	
	public void setView (Chromosome c, int start, int end) {
		if (c == chromosome) {
			showView = true;
			viewStart = start;
			viewEnd = end;
			repaint();
		}
		else {
			if (showView) {
				showView = false;
				repaint();
			}
		}
	}

	private class PanelListener implements MouseListener {

		public void mouseClicked(MouseEvent me) {
			// Figure out where on the chromosome the mouse was clicked
			int midPoint = (int)(maxLen * ((double)me.getX()/getWidth()));
			if (midPoint > chromosome.length()) 
				midPoint = chromosome.length();
			
			viewer.application().displayChromosome(chromosome,midPoint);
		}

		public void mousePressed(MouseEvent arg0) {
		}

		public void mouseReleased(MouseEvent arg0) {
		}

		public void mouseEntered(MouseEvent arg0) {
			viewer.setInfo(chromosome);
		}

		public void mouseExited(MouseEvent arg0) {
		}
	}
	
}
