/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Displays.ScatterPlot;

import java.awt.Color;
import java.awt.Image;

import javax.swing.JPanel;

import java.awt.Graphics;
import java.awt.image.BufferedImage;

import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;

public class ScatterPlotPanel extends JPanel {
	
	private static double log2 = Math.log(2);
	private double [] signalValues;
	private double [] controlValues;
	private double highest;
	private double lowest;
	private String name;
	
	public ScatterPlotPanel (DataCollection c, DataStore d) {
		setBackground(Color.WHITE);
		
		// I'm going to make this always scale to the whole collection even
		// when you're only viewing a subset.  If you don't do this then you
		// can't compare different plots.
		
		name = d.name();
		Probe [] probes = c.probeSet().getAllProbes();
		signalValues = new double [probes.length];
		controlValues = new double [probes.length];
		
		for (int i=0;i<probes.length;i++) {
			signalValues[i] = Math.log(d.getSignalValueForProbe(probes[i]))/log2;
			controlValues[i] = Math.log(d.getControlValueForProbe(probes[i]))/log2;
			if (i==0) {
				highest = signalValues[0];
				lowest = signalValues[0];
			}
			
			if (signalValues[i]>highest) highest=signalValues[i];
			if (signalValues[i]<lowest) lowest=signalValues[i];
			if (controlValues[i]>highest) highest=controlValues[i];
			if (controlValues[i]<lowest) lowest=controlValues[i];
			
			
		}
		
		// We now need to do the whole thing again, just using the active
		// probes this time (for the display).
		
		probes = c.probeSet().getActiveProbes();
		signalValues = new double [probes.length];
		controlValues = new double [probes.length];
		
		for (int i=0;i<probes.length;i++) {
			signalValues[i] = Math.log(d.getSignalValueForProbe(probes[i]))/log2;
			controlValues[i] = Math.log(d.getControlValueForProbe(probes[i]))/log2;
		}
		
	}

	public Image getCurrentView() {
		BufferedImage b = new BufferedImage(getWidth(),getHeight(),BufferedImage.TYPE_INT_RGB);
		Graphics g = b.getGraphics();			
		this.paint(g);
		return b;
	}
	
	public void paintComponent (Graphics g) {
		super.paintComponent(g);
		
		g.setColor(Color.RED);
		for (int i=0;i<signalValues.length;i++) {
			int [] coords = getCoordinates(signalValues[i],controlValues[i]);
//			if (i%10000 == 0) {
//				System.out.println("S="+signalValues[i]+" C="+controlValues[i]+" highest="+highest+" lowest="+lowest+" height="+getHeight()+" width="+getWidth()+" x="+coords[0]+" y="+coords[1]);
//			}
			g.fillRect(coords[0]-1,coords[1]-1,2,2);
		}

		//Draw the axes and labels
		
		g.setColor(Color.LIGHT_GRAY);

		// Diagonal
		g.drawLine(20,getHeight()-20,getWidth()-5,10);

		// Box around plot
		g.drawRect(20,10,getWidth()-25,getHeight()-30);
		
		g.setColor(Color.BLACK);
		
		//X-axis
		g.drawLine(20,getHeight()-20,getWidth()-5,getHeight()-20);
		
		//Y-axis
		g.drawLine(20,10,20,getHeight()-20);
		
		// Y axis label
		g.drawString("Sig",24,20);
		
		// X axis label
		g.drawString("Con",24,getHeight()-10);
				
		// Filename at top
		g.drawString(name,2,10);
			
	}
	
	private int [] getCoordinates (double s, double c) {
		int [] r = new int [2];
				
		// The x value is offset by 20 at the left and 5 at the right
		r[0] = (int)(20+ (((double)getWidth()-25)  *  ((c-lowest)/(highest-lowest))));
		
		// The y value is offset by 10 at top and bottom
		r[1] = (int) ((getHeight()-20) -  (((double)getHeight()-30)* ((s-lowest)/(highest-lowest))));
		
		return r;
	}
}
